package com.gzkx.commonfunction.interceptor;
import org.apache.ibatis.executor.statement.StatementHandler;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.mapping.ParameterMapping;
import org.apache.ibatis.plugin.*;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.reflection.SystemMetaObject;
import org.apache.ibatis.session.Configuration;
import org.apache.ibatis.type.TypeHandlerRegistry;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.slf4j.MDC;

import java.sql.Connection;
import java.util.List;
import java.util.Date;
import java.util.Locale;
import java.util.Properties;
import java.text.DateFormat;

@Intercepts({
        @Signature(type = StatementHandler.class, method = "prepare", args = {Connection.class, Integer.class})
})
public class SqlLoggerInterceptor implements Interceptor {

    private static final Logger sqlLog = LogManager.getLogger(SqlLoggerInterceptor.class);

    @Override
    public Object intercept(Invocation invocation) throws Throwable {

        long startTime = System.currentTimeMillis();

        try {
            // 执行原始方法
            return invocation.proceed();
        } finally {
            long endTime = System.currentTimeMillis();
            long costTime = endTime - startTime;
            // 这里可以获取到 SQL 语句
            StatementHandler statementHandler = (StatementHandler) invocation.getTarget();
            BoundSql boundSql = statementHandler.getBoundSql();
            String sql = boundSql.getSql();

            // 获取参数
            Object parameterObject = boundSql.getParameterObject();
            List<ParameterMapping> parameterMappings = boundSql.getParameterMappings();

            // 使用MetaObject获取Configuration对象
            MetaObject metaStatementHandler = SystemMetaObject.forObject(statementHandler);
            Configuration configuration = (Configuration) metaStatementHandler.getValue("delegate.configuration");
            TypeHandlerRegistry typeHandlerRegistry = configuration.getTypeHandlerRegistry();
            if (parameterObject != null && parameterMappings != null) {
                MetaObject metaObject = configuration.newMetaObject(parameterObject);
                for (ParameterMapping parameterMapping : parameterMappings) {
                    String propertyName = parameterMapping.getProperty();
                    Object value;
                    if (boundSql.hasAdditionalParameter(propertyName)) {
                        value = boundSql.getAdditionalParameter(propertyName);
                    } else if (typeHandlerRegistry.hasTypeHandler(parameterObject.getClass())) {
                        value = parameterObject;
                    } else {
                        value = metaObject.getValue(propertyName);
                    }
                    sql = replaceParameter(sql, value);
                }
            }
            MDC.put("sqlNow", sql);
            MDC.put("costTime", String.valueOf(costTime));
            sqlLog.info("执行SQL: {} | 执行耗时: {} ms", sql, costTime);
            MDC.clear();
        }

    }

    @Override
    public Object plugin(Object target) {
        return Plugin.wrap(target, this);
    }

    @Override
    public void setProperties(Properties properties) {
        // 配置属性，可以根据需要使用
    }

    private String replaceParameter(String sql, Object value) {
        if (value == null) {
            return sql.replaceFirst("\\?", "NULL");
        } else {
            if (value instanceof String) {
                value = "'" + value + "'";
            } else if (value instanceof Date) {
                DateFormat dateFormat = DateFormat.getDateTimeInstance(DateFormat.DEFAULT, DateFormat.DEFAULT, Locale.ENGLISH);
                value = "'" + dateFormat.format(value) + "'";
            }
            return sql.replaceFirst("\\?", value.toString());
        }
    }

}
