package com.gzkx.commonfunction.utils;

import okhttp3.*;
import okio.BufferedSink;
import okio.Okio;
import okio.Source;

import java.io.File;
import java.io.IOException;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;


public class OkHttpUtil {

    private static final MediaType JSON = MediaType.parse("application/json; charset=utf-8");
    private static final MediaType OCTET_STREAM = MediaType.parse("application/octet-stream");
    private static final OkHttpClient client = new OkHttpClient.Builder()
            .connectTimeout(30, TimeUnit.SECONDS)
            .readTimeout(60, TimeUnit.SECONDS)
            .writeTimeout(60, TimeUnit.SECONDS)
            .build();

    // GET 请求
    public static String get(String url, Map<String, String> params) throws IOException {
        HttpUrl.Builder urlBuilder = Objects.requireNonNull(HttpUrl.parse(url)).newBuilder();
        if (params != null) {
            params.forEach(urlBuilder::addQueryParameter);
        }
        Request request = new Request.Builder().url(urlBuilder.build()).build();
        return executeRequest(request);
    }

    // POST 表单请求
    public static String postForm(String url, Map<String, String> formData) throws IOException {
        FormBody.Builder builder = new FormBody.Builder();
        if (formData != null) {
            formData.forEach(builder::add);
        }
        Request request = new Request.Builder()
                .url(url)
                .post(builder.build())
                .build();
        return executeRequest(request);
    }

    // POST JSON 请求
    public static String postJson(String url, String json) throws IOException {
        RequestBody body = RequestBody.create(json, JSON);
        Request request = new Request.Builder()
                .url(url)
                .post(body)
                .build();
        return executeRequest(request);
    }

    // 文件上传
    public static String uploadFile(String url, File file, Map<String, String> formParams) throws IOException {
        MultipartBody.Builder builder = new MultipartBody.Builder()
                .setType(MultipartBody.FORM);

        // 添加文件
        builder.addFormDataPart("file", file.getName(),
                RequestBody.create(file, OCTET_STREAM));

        // 添加其他参数
        if (formParams != null) {
            formParams.forEach(builder::addFormDataPart);
        }

        Request request = new Request.Builder()
                .url(url)
                .post(builder.build())
                .build();
        return executeRequest(request);
    }

    // 文件下载
    public static void downloadFile(String url, String savePath) throws IOException {
        Request request = new Request.Builder().url(url).build();
        try (Response response = client.newCall(request).execute()) {
            File destFile = new File(savePath);
            try (Source source = Objects.requireNonNull(response.body()).source();
                 BufferedSink sink = Okio.buffer(Okio.sink(destFile))) {
                sink.writeAll(source);
            }
        }
    }

    // PUT 请求
    public static String put(String url, String json) throws IOException {
        RequestBody body = RequestBody.create(json, JSON);
        Request request = new Request.Builder()
                .url(url)
                .put(body)
                .build();
        return executeRequest(request);
    }

    // DELETE 请求
    public static String delete(String url) throws IOException {
        Request request = new Request.Builder()
                .url(url)
                .delete()
                .build();
        return executeRequest(request);
    }

    private static String executeRequest(Request request) throws IOException {
        try (Response response = client.newCall(request).execute()) {
            return Objects.requireNonNull(response.body()).string();
        }
    }

}
