package com.gzkx.commonfunction.utils;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.gzkx.commonfunction.model.ValidationResult;
import com.gzkx.commonfunction.model.ValidationRule;
import org.apache.commons.lang3.math.NumberUtils;
import java.math.BigDecimal;
import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

public class ParamsRulesCheckUtil {

    private static final ObjectMapper mapper = new ObjectMapper();

    /**
     * 执行参数校验
     * @param jsonStr 待校验的JSON字符串
     * @param rules 从数据库查询的校验规则列表（需按param_name分组）
     * @return 校验结果对象包含是否通过和错误详情
     */
    public static ValidationResult validate(String jsonStr,
                                            List<ValidationRule> rules) {
        ValidationResult result = new ValidationResult();
        try {
            JsonNode rootNode = mapper.readTree(jsonStr);
            for (ValidationRule rule : rules) {
                String paramName = rule.getParamName();
                JsonNode valueNode = rootNode.get(paramName);
                validateRule(paramName, valueNode, rule, result);
            }
        } catch (Exception e) {
            result.addError("JSON_PARSE_ERROR", "JSON解析失败: " + e.getMessage());
        }
        return result;
    }

    private static void validateRule(String paramName, JsonNode valueNode,
                                     ValidationRule rule, ValidationResult result) {
        // 必填校验
        if (rule.isRequired() && (valueNode == null || valueNode.isNull())) {
            result.addError(paramName, "必填参数缺失");
            return;
        }
        // 非必填为空直接跳过后续校验
        if (valueNode == null) return;

        // 类型校验
        if (!validateType(valueNode, rule.getParamType())) {
            result.addError(paramName, "类型错误，期望类型：" + rule.getParamType());
            return;
        }

        String value = valueNode.asText();

        // 正则校验
        if (rule.getFormatRule() != null && !Pattern.compile(rule.getFormatRule()).matcher(value).matches()) {
            result.addError(paramName, "格式不符合要求，正则规则：" + rule.getFormatRule());
        }

        // 数值范围校验
        if (NumberUtils.isParsable(value)) {
            BigDecimal numValue = new BigDecimal(value);
            if (rule.getMinValue() != null && numValue.compareTo(rule.getMinValue()) < 0) {
                result.addError(paramName, "数值小于最小值：" + rule.getMinValue());
            }
            if (rule.getMaxValue() != null && numValue.compareTo(rule.getMaxValue()) > 0) {
                result.addError(paramName, "数值超过最大值：" + rule.getMaxValue());
            }
        }

        // 枚举值校验
        if (rule.getAllowedValues() != null && !rule.getAllowedValues().isEmpty()) {
            if (!rule.getAllowedValues().contains(value)) {
                result.addError(paramName, "值不在允许范围内，允许值：" + String.join(",", rule.getAllowedValues()));
            }
        }
    }

    private static boolean validateType(JsonNode node, String expectedType) {
        return switch (expectedType.toLowerCase()) {
            case "string" -> node.isTextual();
            case "integer" -> node.isInt() || node.isLong();
            case "number" -> node.isNumber();
            case "boolean" -> node.isBoolean();
            default -> true; // 未知类型不校验
        };
    }

    public static void main(String[] args) {
        String jsonStr = "{\"userId\":\"user_003\",\"pageSize\":\"10\",\"phone\":\"18581383125\"}";
        List<ValidationRule> rules = List.of(
                new ValidationRule("pageNum", true, "integer", null, null, null, null),
                new ValidationRule("pageSize", true, "integer", null, null, null, null),
                new ValidationRule("phone", false, "string", "^1[3-9]\\\\d{9}$", null, null, null)
        );
        System.out.println("JSON串: " + jsonStr);
        System.out.println("规则对象: " + rules);
        ValidationResult result = validate(jsonStr, rules);
        System.out.println(result.getErrors().toString());
        if (result.isSuccess()) {
            System.out.println("校验通过");
        } else {
            System.out.println("校验失败");
            throw new RuntimeException(result.getErrors().toString());
        }
    }

}
