package com.gzkx.commonfunction.utils.sso;

import io.minio.GetObjectArgs;
import io.minio.MinioClient;
import io.minio.errors.MinioException;

import java.io.InputStream;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

/**
 * Key 工具类 - 从 MinIO 读取公私钥并转换为 Java 密钥对象
 */
public class KeyUtils {

    /**
     * 从 MinIO 获取公钥对象
     */
    public static PublicKey getPublicKeyFromMinio(
            String minioEndpoint,
            String accessKey,
            String secretKey,
            String bucketName,
            String objectName) throws Exception {
        byte[] keyBytes = downloadFromMinio(minioEndpoint, accessKey, secretKey, bucketName, objectName);
        return parsePublicKey(keyBytes);
    }

    /**
     * 从 MinIO 获取私钥对象
     */
    public static PrivateKey getPrivateKeyFromMinio(
            String minioEndpoint,
            String accessKey,
            String secretKey,
            String bucketName,
            String objectName) throws Exception {
        byte[] keyBytes = downloadFromMinio(minioEndpoint, accessKey, secretKey, bucketName, objectName);
        return parsePrivateKey(keyBytes);
    }

    /**
     * 解析公钥
     */
    private static PublicKey parsePublicKey(byte[] keyBytes) throws Exception {
        X509EncodedKeySpec spec = new X509EncodedKeySpec(keyBytes);
        KeyFactory factory = KeyFactory.getInstance("RSA");
        return factory.generatePublic(spec);
    }

    /**
     * 解析私钥
     */
    private static PrivateKey parsePrivateKey(byte[] keyBytes) throws Exception {
        PKCS8EncodedKeySpec spec = new PKCS8EncodedKeySpec(keyBytes);
        KeyFactory factory = KeyFactory.getInstance("RSA");
        return factory.generatePrivate(spec);
    }

    /**
     * 从 MinIO 下载文件为字节数组
     */
    private static byte[] downloadFromMinio(
            String endpoint,
            String accessKey,
            String secretKey,
            String bucketName,
            String objectName) throws Exception {
        try {
            MinioClient minioClient = MinioClient.builder()
                    .endpoint(endpoint)
                    .credentials(accessKey, secretKey)
                    .build();

            try (InputStream stream = minioClient.getObject(
                    GetObjectArgs.builder()
                            .bucket(bucketName)
                            .object(objectName)
                            .build())) {
                return stream.readAllBytes();
            }
        } catch (MinioException e) {
            throw new Exception("从 MinIO 下载密钥失败: " + e.getMessage(), e);
        }
    }
}
